
################################################################################
#                                       
#  makefile:  PerfmonAgent project   
#                                       
# $Id: PerfmonAgent_zos.make,v 1.1 2009/08/26 14:46:18 jwest Exp $                                  
################################################################################


#-----------------------------------
#
#   Customizable section
#
#-----------------------------------
TARGET         :=  LinuxAgent
TARGETDIR      :=  ../../../../bin


#-----------------------------------
#   list of additional INCLUDE directories to search headers from
#      (separated by blanks)
#-----------------------------------

INCLUDEDIRS    :=  . ../resutils


#-----------------------------------
#   list of additional libraries to be linked with
#      (separated by blanks)
#-----------------------------------

LIBS       :=  dl pthread resutils stdc++

#-----------------------------------
#   list of additional library directories to search from
#      (separated by blanks)
#-----------------------------------

LIBDIRS    :=  ../../../../lib


#-----------------------------------
#
#   Compiler options
#
#      -g       produce debug info
#      -Wall    enable all types of warnings
#      -O3      highest level of code optimization
#      -fPIC    generate position-independent code (PIC).
#               This is required for building shared library.
#      -c       compile only
#
#   Linking options
#
#       -Wl       pass options to linker
#       -shared   produce shared library
#       -L        add the given directory to the search path for libraries
#
#-----------------------------------

DEFINES    := -D_TPTP_DIAGNOSTIC_

UNAME = $(shell uname -s)
MNAME = $(shell uname -m)

ifeq ($(UNAME), Linux)
	ifeq ($(MNAME), s390x)
		DEFINES += -D_LINUX_390 -DLINUX_S390
	else
		ifeq ($(MNAME), s390)
			DEFINES += -D_LINUX_390 -DLINUX_S390
		else
			DEFINES += -D_LINUX_X86 -DLINUX
		endif
	endif
endif

CFLAGS     :=  $(C_FLAGS) $(DEBUGABLE) -Wall $(OPTIMIZABLE) -fPIC $(addprefix -I,$(INCLUDEDIRS)) $(DEFINES)

CXXFLAGS   :=  $(CFLAGS)

LINKOPTS   :=  $(L_FLAGS) $(DEBUGABLE) -o $(TARGETDIR)/$(TARGET) $(addprefix -L,$(LIBDIRS)) $(DEFINES)


#-----------------------------------
#   general commands and options
#-----------------------------------

CC          :=  gcc
CXX         :=  $(CC)

SOURCE      :=  $(wildcard *.c) $(wildcard *.cpp)

OBJS        :=  $(patsubst %.c,%.o,$(patsubst %.cpp,%.o,$(SOURCE)))

REMOVEFILE :=  rm -f
CREATELINK :=  ln -sf


#-----------------------------------
#   "make" rules
#-----------------------------------

.PHONY : default everything objs clean veryclean rebuild $(TARGET)


default : $(TARGET)


objs : $(OBJS)


clean :
	$(REMOVEFILE) *.o

veryclean: clean
	$(REMOVEFILE) $(TARGETDIR)/$(TARGET)

rebuild: veryclean everything


$(TARGET) : $(OBJS)
	$(CC) $(LINKOPTS) $(OBJS) $(addprefix -l,$(LIBS))

