 
################################################################################
#
#  makefile:  PerfmonAgent sample project
#
# $Id: PerfmonAgent.make,v 1.3 2009/01/16 14:34:45 jwest Exp $
################################################################################


#-----------------------------------
#
#  Customizable section
#
#-----------------------------------

TARGETDIR      :=  ../../bin
LIBDIR         :=  ../../lib
LIBDIRS        :=  ../../lib
PERFLINLIB     :=  $(LIBDIR)/libperflin.so
SUPPORTLIB     :=  $(LIBDIR)/libresutils.a
RESUTILOBJS    :=  library.o clock.o hashtable.o random.o streq.o thread.o mutex.o

INCLUDES       :=  ./resutils ../../include

LIBS           :=  dl pthread resutils processControlUtil tptpAgentBase tptpUtils tptpConfig tptpLogUtils transportSupport hcclco hcclsm hccls

UNAME = $(shell uname -s)
MNAME = $(shell uname -m)

ifeq ($(UNAME), Linux)
	ifeq ($(MNAME), s390x)
		DEFINES += -D_LINUX_390 -DLINUX_S390
	else
		ifeq ($(MNAME), s390)
			DEFINES += -D_LINUX_390 -DLINUX_S390
		else
			DEFINES += -D_LINUX_X86 -DLINUX
		endif
	endif
endif

.PHONY: default everything clean veryclean rebuild $(TARGETDIR)/PerfmonAgent


default: $(TARGETDIR)/PerfmonAgent

clean:
	rm -f *.o


veryclean: clean
	rm -f $(TARGETDIR)/PerfmonAgent $(PERFLINLIB) $(SUPPORTLIB)


rebuild: veryclean everything


$(TARGETDIR)/PerfmonAgent: PerfmonAgent/PerfmonAgent.cpp PerfmonAgent/PerfmonAgent.h $(PERFLINLIB) $(SUPPORTLIB)
	g++ $(DEBUGABLE) -Wall $(DEFINES) $(addprefix -I,$(INCLUDES)) -o $(TARGETDIR)/PerfmonAgent PerfmonAgent/PerfmonAgent.cpp $(addprefix -L,$(LIBDIRS)) $(addprefix -l,$(LIBS))

#-----------------------------------
#
#  Resutils
#
#-----------------------------------

$(SUPPORTLIB): $(RESUTILOBJS)
	ar -rs $(SUPPORTLIB) $(RESUTILOBJS)

hashtable.o: resutils/hashtable.c resutils/hashtable.h
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/hashtable.c

clock.o: resutils/clock.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/clock.c

library.o: resutils/library.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/library.c

mutex.o: resutils/mutex.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/mutex.c

random.o: resutils/random.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/random.c

streq.o: resutils/streq.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/streq.c

thread.o: resutils/thread.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils resutils/thread.c


#-----------------------------------
#
#  Perflin
#
#-----------------------------------

$(PERFLINLIB): perflin.o $(SUPPORTLIB)
	gcc $(DEBUGABLE) -shared -I./resutils $(addprefix -L,$(LIBDIRS)) -shared -o $(PERFLINLIB) perflin.o

perflin.o: perflin/perflin.c
	gcc $(DEBUGABLE) -fPIC -c $(DEFINES) -I./resutils perflin/perflin.c
